#include <stdlib.h>
#include <stdio.h>
#include "aswm/mapper/mapper.h"

AswmTile* aswm_alloc_tile() {
	AswmTile* tile = malloc(sizeof(AswmTile));
	for(int i = 0; i < 2; i++)
		tile->children[i] = nullptr;
	return tile;
}

AswmTile* aswm_alloc_root_tile(Display* display, Window window) {
	int screen_number = XDefaultScreen(display);
	AswmTile* tile = aswm_alloc_tile();

	tile->has_children = false;
	tile->x = 0;
	tile->y = 0;
	tile->free = true;
	tile->width = XDisplayWidth(display, screen_number);
	tile->height = XDisplayHeight(display, screen_number);

	return tile;
}

void aswm_free_tile(AswmTile* tile) {
	for(int i = 0; i < 2; i++)
		aswm_free_tile(tile->children[i]);
	if(tile != nullptr)
		free(tile);
}

/**
 * Returns the free tile built in the split process.
 */
AswmTile* h_split(AswmTile* tile);

AswmTile* aswm_map_new_window(Display* display, Window window, AswmTile* tile) {
	if(tile->has_children) {
		// The tile is necessarily considered occupied
		return aswm_map_new_window(display, window, tile->children[1]);
	}
	if(tile->free) {
		// Leef of the tile tree, the window can be inserted there
		tile->free = false;
		tile->display = display;
		tile->window = window;

		// Automatically reconfigure position and size of the window
		XWindowChanges changes;
		changes.x = tile->x;
		changes.y = tile->y;
		changes.width = tile->width;
		changes.height = tile->height;

		XConfigureWindow(display, window, CWX | CWY | CWWidth | CWHeight, &changes);
		return tile;
	}
	// Leef of tile tree, but occupied. Split the tile and map the window to the
	// new tile.
	return aswm_map_new_window(display, window, h_split(tile));
}

AswmTile* h_split(AswmTile* tile) {
	tile->has_children = true;
	for(int i = 0; i < 2; i++) {
		tile->children[i] = aswm_alloc_tile();
		tile->children[i]->has_children = false;
		tile->children[i]->free = true;
		tile->children[i]->x = tile->x;
		tile->children[i]->width = tile->width;
		tile->children[i]->height = tile->height / 2;
	}
	tile->children[0]->y = tile->y;
	tile->children[1]->y = tile->y + tile->children[0]->height;

	aswm_map_new_window(tile->display, tile->window, tile->children[0]);

	return tile->children[1];
}
